#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" API for the ask-owl Microservice """


from rdflib import Graph
from baseblock import BaseObject

from ask_owl.svc import LoadOntologyModel
from ask_owl.svc import QueryOntologyModel
from ask_owl.dto import QueryResultType


class AskOwlAPI(BaseObject):
    """ API for the ask-owl Microservice """

    def __init__(self,
                 ontology_name: str):
        """ Change History

        Created:
            25-May-2022
            craig@grafflr.ai
            *   https://github.com/grafflr/ask-owl/issues/1

        Args:
            ontology_name (str): the name of the Ontology (OWL) model
        """
        BaseObject.__init__(self, __name__)
        self._model_loader = LoadOntologyModel(ontology_name)
        self._execute_query = QueryOntologyModel(
            self._model_loader.graph()).process

    def graph(self) -> Graph:
        return self._model_loader.graph()

    def by_subject_predicate(self,
                             predicate: str,
                             subject: str) -> list:
        """ Retrieve a list of values by custom subject and custom predicate

        Args:
            predicate (str): the predicate name
            subject (str): the subject name

        Returns:
            list: a list of values for this predicate
        """
        sparql = "SELECT ?b WHERE { #PREFIX:#SUBJECT rdf:type owl:Class ; #PREFIX:#PREDICATE ?b }"
        sparql = sparql.replace('#PREFIX', self._model_loader.prefix)
        sparql = sparql.replace('#PREDICATE', predicate)
        sparql = sparql.replace('#SUBJECT', subject)

        d_results = self._execute_query(
            sparql=sparql,
            result_type=QueryResultType.LIST_OF_STRINGS,
        )

        return d_results

    def by_predicate(self,
                     predicate: str,
                     reverse: bool = False) -> list:
        """ Retrieve a list of values by custom predicate

        Args:
            predicate (str): the predicate name
            reverse (bool, optional): reverses the subject/object order. Defaults to False.
                if "?x implies ?y" and reverse=False
                    the results will be { ?x: [?y-1, ?y-2, ..., ?y-N]}
                if "?x implies ?y" and reverse=True
                    the results will be { ?y-1: [x], ?y-2: [x], ?y-N: [x]}

        Returns:
            dict: a dict of values for this predicate
        """
        sparql = "SELECT ?a ?b WHERE { ?a #PREFIX:#PREDICATE ?b }"
        sparql = sparql.replace('#PREFIX', self._model_loader.prefix)
        sparql = sparql.replace('#PREDICATE', predicate)

        d_results = self._execute_query(
            sparql=sparql,
            reverse=reverse,
            result_type=QueryResultType.DICT_OF_STR2LIST,
        )

        return d_results

    def labels(self) -> list:
        """ Retrieve rdfs:label values from the Graph

        Returns:
            list: a list of labels
        """
        sparql = "SELECT ?a WHERE { ?x rdfs:label ?a }"
        return self._execute_query(sparql, QueryResultType.LIST_OF_STRINGS)

    def see_also(self) -> list:
        """ Retrieve rdfs:seeAlso values from the Graph

        Returns:
            list: a list of see-also values
        """
        sparql = "SELECT ?a WHERE { ?x rdfs:seeAlso ?a }"
        return self._execute_query(sparql, QueryResultType.LIST_OF_STRINGS)

    def backward_compatible_with(self) -> list:
        """ Retrieve owl:backwardCompatibleWith values from the Graph

        Returns:
            list: a list of see-also values
        """
        sparql = "SELECT ?a WHERE { ?x rdfs:seeAlso ?a }"
        return self._execute_query(sparql, QueryResultType.LIST_OF_STRINGS)

    def types(self) -> list:
        """ Retrieve rdf:type values from the Graph

        Returns:
            list: a list of labels
        """
        sparql = "SELECT ?a WHERE { ?a rdf:type ?x }"
        return self._execute_query(sparql, QueryResultType.LIST_OF_STRINGS)
